@@ -1,6 +1,5 @@
 /*
- * Copyright (C) 2004-2009  Internet Systems Consortium, Inc. ("ISC")
- * Copyright (C) 2000-2003  Internet Software Consortium.
+ * Copyright (C) Internet Systems Consortium, Inc. ("ISC")
  *
  * Permission to use, copy, modify, and/or distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -368,7 +367,7 @@ ns_lwdmanager_create(isc_mem_t *mctx, const cfg_obj_t *lwres,
 
 			dns_fixedname_init(&fname);
 			name = dns_fixedname_name(&fname);
-			isc_buffer_init(&namebuf, searchstr,
+			isc_buffer_constinit(&namebuf, searchstr,
 					strlen(searchstr));
 			isc_buffer_add(&namebuf, strlen(searchstr));
 			result = dns_name_fromtext(name, &namebuf,
@@ -602,7 +601,7 @@ listener_copysock(ns_lwreslistener_t *oldlistener,
 
 static isc_result_t
 listener_startclients(ns_lwreslistener_t *listener) {
-	ns_lwdclientmgr_t *cm;
+	ns_lwdclientmgr_t *cm, *next;
 	unsigned int i;
 	isc_result_t result;
 
@@ -626,6 +625,7 @@ listener_startclients(ns_lwreslistener_t *listener) {
 	LOCK(&listener->lock);
 	cm = ISC_LIST_HEAD(listener->cmgrs);
 	while (cm != NULL) {
+		next = ISC_LIST_NEXT(cm, link);
 		result = ns_lwdclient_startrecv(cm);
 		if (result != ISC_R_SUCCESS)
 			isc_log_write(ns_g_lctx, NS_LOGCATEGORY_GENERAL,
@@ -633,7 +633,7 @@ listener_startclients(ns_lwreslistener_t *listener) {
 				      "could not start lwres "
 				      "client handler: %s",
 				      isc_result_totext(result));
-		cm = ISC_LIST_NEXT(cm, link);
+		cm = next;
 	}
 	UNLOCK(&listener->lock);
 
