@@ -1,6 +1,6 @@
 #!/bin/sh
 #
-# Copyright (C) 2008-2012  Internet Systems Consortium, Inc. ("ISC")
+# Copyright (C) Internet Systems Consortium, Inc. ("ISC")
 #
 # Permission to use, copy, modify, and/or distribute this software for any
 # purpose with or without fee is hereby granted, provided that the above
@@ -14,13 +14,14 @@
 # OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 # PERFORMANCE OF THIS SOFTWARE.
 
-# $Id$
-
 #
 # Clean up after resolver tests.
 #
+rm -f */named.conf
 rm -f */named.memstats
-rm -f dig.out dig.*.out.*
+rm -f */named.run
+rm -f */ans.run
+rm -f dig.out dig.out.* dig.ns*.out*
 rm -f dig.*.foo.*
 rm -f dig.*.bar.*
 rm -f dig.*.prime.*
@@ -30,4 +31,5 @@ rm -f ns6/example.net.db.signed ns6/example.net.db
 rm -f ns6/dsset-example.net. ns6/example.net.db.signed.jnl
 rm -f ns6/to-be-removed.tld.db ns6/to-be-removed.tld.db.jnl
 rm -f ns7/server.db ns7/server.db.jnl
-rm -f random.data
+rm -f sample.out.*.test*
+rm -f .digrc
