@@ -1,7 +1,6 @@
 #!/usr/bin/perl -w
 #
-# Copyright (C) 2004-2008, 2010-2017  Internet Systems Consortium, Inc. ("ISC")
-# Copyright (C) 2001  Internet Software Consortium.
+# Copyright (C) Internet Systems Consortium, Inc. ("ISC")
 #
 # Permission to use, copy, modify, and/or distribute this software for any
 # purpose with or without fee is hereby granted, provided that the above
@@ -15,8 +14,6 @@
 # OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 # PERFORMANCE OF THIS SOFTWARE.
 
-# $Id$
-
 # Framework for starting test servers.
 # Based on the type of server specified, check for port availability, remove
 # temporary files, start the server, and verify that the server is running.
@@ -27,38 +24,60 @@ use Cwd;
 use Cwd 'abs_path';
 use Getopt::Long;
 
-# Option handling
-#   --noclean test [server [options]]
+# Usage:
+#   perl start.pl [--noclean] [--restart] [--port port] test [server [options]]
 #
-#   --noclean - Do not cleanup files in server directory
-#   test - name of the test directory
-#   server - name of the server directory
-#   options - alternate options for the server
-#             NOTE: options must be specified with '-- "<option list>"',
-#              for instance: start.pl . ns1 -- "-c n.conf -d 43"
-#             ALSO NOTE: this variable will be filled with the
-#		contents of the first non-commented/non-blank line of args
-#		in a file called "named.args" in an ns*/ subdirectory only
-#		the FIRST non-commented/non-blank line is used (everything
-#		else in the file is ignored. If "options" is already set,
-#		then "named.args" is ignored.
+#   --noclean       Do not cleanup files in server directory.
+#
+#   --restart       Indicate that the server is being restarted, so get the
+#                   server to append output to an existing log file instead of
+#                   starting a new one.
+#
+#   --port port     Specify the default port being used by the server to answer
+#                   queries (default 5300).  This script will interrogate the
+#                   server on this port to see if it is running. (Note: for
+#                   "named" nameservers, this can be overridden by the presence
+#                   of the file "named.port" in the server directory containing
+#                   the number of the query port.)
+#
+#   test            Name of the test directory.
+#
+#   server          Name of the server directory.  This will be of the form
+#                   "nsN" or "ansN", where "N" is an integer between 1 and 8.
+#                   If not given, the script will start all the servers in the
+#                   test directory.
+#
+#   options         Alternate options for the server.
+#
+#                   NOTE: options must be specified with '-- "<option list>"',
+#                   for instance: start.pl . ns1 -- "-c n.conf -d 43"
+#
+#                   ALSO NOTE: this variable will be filled with the contents
+#                   of the first non-commented/non-blank line of args in a file
+#                   called "named.args" in an ns*/ subdirectory. Only the FIRST
+#                   non-commented/non-blank line is used (everything else in
+#                   the file is ignored). If "options" is already set, then
+#                   "named.args" is ignored.
 
-my $usage = "usage: $0 [--noclean] [--restart] test-directory [server-directory [server-options]]";
+my $usage = "usage: $0 [--noclean] [--restart] [--port <port>] test-directory [server-directory [server-options]]";
 my $noclean = '';
 my $restart = '';
-GetOptions('noclean' => \$noclean, 'restart' => \$restart);
+my $queryport = 5300;
+
+GetOptions('noclean' => \$noclean, 'restart' => \$restart, 'port=i' => \$queryport) or die "$usage\n";
+
 my $test = $ARGV[0];
 my $server = $ARGV[1];
 my $options = $ARGV[2];
 
 if (!$test) {
-	print "$usage\n";
+	die "$usage\n";
 }
 if (!-d $test) {
-	print "No test directory: \"$test\"\n";
+	die "No test directory: \"$test\"\n";
 }
 if ($server && !-d "$test/$server") {
-	print "No server directory: \"$test/$server\"\n";
+	die "No server directory: \"$test/$server\"\n";
 }
 
 # Global variables
@@ -96,7 +115,6 @@ if ($server) {
 	foreach $name(@ns, @lwresd, @ans) {
 		&start_server($name);
 		&verify_server($name) if ($name =~ /^ns/);
-		
 	}
 }
 
@@ -105,7 +123,7 @@ if ($server) {
 sub check_ports {
 	my $server = shift;
 	my $options = "";
-	my $port = 5300;
+	my $port = $queryport;
 	my $file = "";
 
 	$file = $testdir . "/" . $server . "/named.port" if ($server);
@@ -182,12 +200,18 @@ sub start_server {
 		} else {
 			$command .= "-m record,size,mctx ";
 			$command .= "-T clienttest ";
-			$command .= "-T nosoa " 
+			$command .= "-T nosoa "
 				if (-e "$testdir/$server/named.nosoa");
-			$command .= "-T noaa " 
+			$command .= "-T noaa "
 				if (-e "$testdir/$server/named.noaa");
+			$command .= "-T noedns "
+				if (-e "$testdir/$server/named.noedns");
 			$command .= "-T dropedns "
 				if (-e "$testdir/$server/named.dropedns");
+			$command .= "-T maxudp512 "
+				if (-e "$testdir/$server/named.maxudp512");
+			$command .= "-T maxudp1460 "
+				if (-e "$testdir/$server/named.maxudp1460");
 			$command .= "-c named.conf -d 99 -g -U 4";
 		}
 		$command .= " -T notcp"
@@ -228,7 +252,7 @@ sub start_server {
 	} elsif ($server =~ /^ans/) {
 		$cleanup_files = "{ans.run}";
                 if (-e "$testdir/$server/ans.py") {
-                        $command = "$PYTHON -u ans.py 10.53.0.$' 5300";
+                        $command = "$PYTHON -u ans.py 10.53.0.$' $queryport";
                 } elsif (-e "$testdir/$server/ans.pl") {
                         $command = "$PERL ans.pl";
                 } else {
@@ -290,7 +314,7 @@ sub start_server {
 sub verify_server {
 	my $server = shift;
 	my $n = $server;
-	my $port = 5300;
+	my $port = $queryport;
 	my $tcp = "+tcp";
 
 	$n =~ s/^ns//;
@@ -312,7 +336,6 @@ sub verify_server {
 		my $return = system("$DIG $tcp +noadd +nosea +nostat +noquest +nocomm +nocmd +noedns -p $port version.bind. chaos txt \@10.53.0.$n > dig.out");
 		last if ($return == 0);
 		if (++$tries >= 30) {
-			print `grep ";" dig.out > /dev/null`;
 			print "I:no response from $server\n";
 			print "I:failed\n";
 			system("$PERL $topdir/stop.pl $testdir");
