@@ -1,6 +1,5 @@
 /*
- * Copyright (C) 2004, 2005, 2007, 2009, 2014  Internet Systems Consortium, Inc. ("ISC")
- * Copyright (C) 1999-2001, 2003  Internet Software Consortium.
+ * Copyright (C) Internet Systems Consortium, Inc. ("ISC")
  *
  * Permission to use, copy, modify, and/or distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -33,7 +32,7 @@ fromtext_afsdb(ARGS_FROMTEXT) {
 	dns_name_t name;
 	isc_boolean_t ok;
 
-	REQUIRE(type == 18);
+	REQUIRE(type == dns_rdatatype_afsdb);
 
 	UNUSED(type);
 	UNUSED(rdclass);
@@ -55,7 +54,8 @@ fromtext_afsdb(ARGS_FROMTEXT) {
 				      ISC_FALSE));
 	dns_name_init(&name, NULL);
 	buffer_fromregion(&buffer, &token.value.as_region);
-	origin = (origin != NULL) ? origin : dns_rootname;
+	if (origin == NULL)
+		origin = dns_rootname;
 	RETTOK(dns_name_fromtext(&name, &buffer, origin, options, target));
 	ok = ISC_TRUE;
 	if ((options & DNS_RDATA_CHECKNAMES) != 0)
@@ -76,7 +76,7 @@ totext_afsdb(ARGS_TOTEXT) {
 	isc_boolean_t sub;
 	unsigned int num;
 
-	REQUIRE(rdata->type == 18);
+	REQUIRE(rdata->type == dns_rdatatype_afsdb);
 	REQUIRE(rdata->length != 0);
 
 	dns_name_init(&name, NULL);
@@ -85,7 +85,7 @@ totext_afsdb(ARGS_TOTEXT) {
 	dns_rdata_toregion(rdata, &region);
 	num = uint16_fromregion(&region);
 	isc_region_consume(&region, 2);
-	sprintf(buf, "%u ", num);
+	snprintf(buf, sizeof(buf), "%u ", num);
 	RETERR(str_totext(buf, target));
 	dns_name_fromregion(&name, &region);
 	sub = name_prefix(&name, tctx->origin, &prefix);
@@ -98,7 +98,7 @@ fromwire_afsdb(ARGS_FROMWIRE) {
 	isc_region_t sr;
 	isc_region_t tr;
 
-	REQUIRE(type == 18);
+	REQUIRE(type == dns_rdatatype_afsdb);
 
 	UNUSED(type);
 	UNUSED(rdclass);
@@ -126,7 +126,7 @@ towire_afsdb(ARGS_TOWIRE) {
 	dns_name_t name;
 	dns_offsets_t offsets;
 
-	REQUIRE(rdata->type == 18);
+	REQUIRE(rdata->type == dns_rdatatype_afsdb);
 	REQUIRE(rdata->length != 0);
 
 	dns_compress_setmethods(cctx, DNS_COMPRESS_NONE);
@@ -154,7 +154,7 @@ compare_afsdb(ARGS_COMPARE) {
 
 	REQUIRE(rdata1->type == rdata2->type);
 	REQUIRE(rdata1->rdclass == rdata2->rdclass);
-	REQUIRE(rdata1->type == 18);
+	REQUIRE(rdata1->type == dns_rdatatype_afsdb);
 	REQUIRE(rdata1->length != 0);
 	REQUIRE(rdata2->length != 0);
 
@@ -182,7 +182,7 @@ fromstruct_afsdb(ARGS_FROMSTRUCT) {
 	dns_rdata_afsdb_t *afsdb = source;
 	isc_region_t region;
 
-	REQUIRE(type == 18);
+	REQUIRE(type == dns_rdatatype_afsdb);
 	REQUIRE(source != NULL);
 	REQUIRE(afsdb->common.rdclass == rdclass);
 	REQUIRE(afsdb->common.rdtype == type);
@@ -201,7 +201,7 @@ tostruct_afsdb(ARGS_TOSTRUCT) {
 	dns_rdata_afsdb_t *afsdb = target;
 	dns_name_t name;
 
-	REQUIRE(rdata->type == 18);
+	REQUIRE(rdata->type == dns_rdatatype_afsdb);
 	REQUIRE(target != NULL);
 	REQUIRE(rdata->length != 0);
 
@@ -229,7 +229,7 @@ freestruct_afsdb(ARGS_FREESTRUCT) {
 	dns_rdata_afsdb_t *afsdb = source;
 
 	REQUIRE(source != NULL);
-	REQUIRE(afsdb->common.rdtype == 18);
+	REQUIRE(afsdb->common.rdtype == dns_rdatatype_afsdb);
 
 	if (afsdb->mctx == NULL)
 		return;
@@ -244,7 +244,7 @@ additionaldata_afsdb(ARGS_ADDLDATA) {
 	dns_offsets_t offsets;
 	isc_region_t region;
 
-	REQUIRE(rdata->type == 18);
+	REQUIRE(rdata->type == dns_rdatatype_afsdb);
 
 	dns_name_init(&name, offsets);
 	dns_rdata_toregion(rdata, &region);
@@ -259,7 +259,7 @@ digest_afsdb(ARGS_DIGEST) {
 	isc_region_t r1, r2;
 	dns_name_t name;
 
-	REQUIRE(rdata->type == 18);
+	REQUIRE(rdata->type == dns_rdatatype_afsdb);
 
 	dns_rdata_toregion(rdata, &r1);
 	r2 = r1;
@@ -275,7 +275,7 @@ digest_afsdb(ARGS_DIGEST) {
 static inline isc_boolean_t
 checkowner_afsdb(ARGS_CHECKOWNER) {
 
-	REQUIRE(type == 18);
+	REQUIRE(type == dns_rdatatype_afsdb);
 
 	UNUSED(name);
 	UNUSED(type);
@@ -290,7 +290,7 @@ checknames_afsdb(ARGS_CHECKNAMES) {
 	isc_region_t region;
 	dns_name_t name;
 
-	REQUIRE(rdata->type == 18);
+	REQUIRE(rdata->type == dns_rdatatype_afsdb);
 
 	UNUSED(owner);
 
