@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006-2008, 2010-2012  Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (C) Internet Systems Consortium, Inc. ("ISC")
  *
  * Permission to use, copy, modify, and/or distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -23,9 +23,11 @@
 #include <isc/buffer.h>
 #include <isc/httpd.h>
 #include <isc/mem.h>
+#include <isc/print.h>
 #include <isc/socket.h>
 #include <isc/string.h>
 #include <isc/task.h>
+#include <isc/time.h>
 #include <isc/util.h>
 
 #include <string.h>
@@ -58,22 +60,9 @@
 #define HTTP_SENDGROW			1024
 #define HTTP_SEND_MAXLEN		10240
 
-/*%
- * HTTP urls.  These are the URLs we manage, and the function to call to
- * provide the data for it.  We pass in the base url (so the same function
- * can handle multiple requests), and a structure to fill in to return a
- * result to the client.  We also pass in a pointer to be filled in for
- * the data cleanup function.
- */
-struct isc_httpdurl {
-	char			       *url;
-	isc_httpdaction_t	       *action;
-	void			       *action_arg;
-	ISC_LINK(isc_httpdurl_t)	link;
-};
-
 #define HTTPD_CLOSE		0x0001 /* Got a Connection: close header */
 #define HTTPD_FOUNDHOST		0x0002 /* Got a Host: header */
+#define HTTPD_KEEPALIVE		0x0004 /* Got a Connection: Keep-Alive */
 
 /*% http client */
 struct isc_httpd {
@@ -87,6 +76,7 @@ struct isc_httpd {
 	 */
 	char			recvbuf[HTTP_RECVLEN]; /*%< receive buffer */
 	isc_uint32_t		recvlen;	/*%< length recv'd */
+	char		       *headers;	/*%< set in process_request() */
 	unsigned int		method;
 	char		       *url;
 	char		       *querystring;
@@ -217,20 +207,12 @@ static isc_result_t process_request(isc_httpd_t *, int);
 static void httpdmgr_destroy(isc_httpdmgr_t *);
 static isc_result_t grow_headerspace(isc_httpd_t *);
 static void reset_client(isc_httpd_t *httpd);
-static isc_result_t render_404(const char *, const char *,
-			       void *,
-			       unsigned int *, const char **,
-			       const char **, isc_buffer_t *,
-			       isc_httpdfree_t **, void **);
-static isc_result_t render_500(const char *, const char *,
-			       void *,
-			       unsigned int *, const char **,
-			       const char **, isc_buffer_t *,
-			       isc_httpdfree_t **, void **);
+
+static isc_httpdaction_t render_404;
+static isc_httpdaction_t render_500;
 
 static void
-destroy_client(isc_httpd_t **httpdp)
-{
+destroy_client(isc_httpd_t **httpdp) {
 	isc_httpd_t *httpd = *httpdp;
 	isc_httpdmgr_t *httpdmgr = httpd->mgr;
 
@@ -256,39 +238,40 @@ isc_result_t
 isc_httpdmgr_create(isc_mem_t *mctx, isc_socket_t *sock, isc_task_t *task,
 		    isc_httpdclientok_t *client_ok,
 		    isc_httpdondestroy_t *ondestroy, void *cb_arg,
-		    isc_timermgr_t *tmgr, isc_httpdmgr_t **httpdp)
+		    isc_timermgr_t *tmgr, isc_httpdmgr_t **httpdmgrp)
 {
 	isc_result_t result;
-	isc_httpdmgr_t *httpd;
+	isc_httpdmgr_t *httpdmgr;
 
 	REQUIRE(mctx != NULL);
 	REQUIRE(sock != NULL);
 	REQUIRE(task != NULL);
 	REQUIRE(tmgr != NULL);
-	REQUIRE(httpdp != NULL && *httpdp == NULL);
+	REQUIRE(httpdmgrp != NULL && *httpdmgrp == NULL);
 
-	httpd = isc_mem_get(mctx, sizeof(isc_httpdmgr_t));
-	if (httpd == NULL)
+	httpdmgr = isc_mem_get(mctx, sizeof(isc_httpdmgr_t));
+	if (httpdmgr == NULL)
 		return (ISC_R_NOMEMORY);
 
-	result = isc_mutex_init(&httpd->lock);
+	result = isc_mutex_init(&httpdmgr->lock);
 	if (result != ISC_R_SUCCESS) {
-		isc_mem_put(mctx, httpd, sizeof(isc_httpdmgr_t));
+		isc_mem_put(mctx, httpdmgr, sizeof(isc_httpdmgr_t));
 		return (result);
 	}
-	httpd->mctx = NULL;
-	isc_mem_attach(mctx, &httpd->mctx);
-	httpd->sock = NULL;
-	isc_socket_attach(sock, &httpd->sock);
-	httpd->task = NULL;
-	isc_task_attach(task, &httpd->task);
-	httpd->timermgr = tmgr; /* XXXMLG no attach function? */
-	httpd->client_ok = client_ok;
-	httpd->ondestroy = ondestroy;
-	httpd->cb_arg = cb_arg;
+	httpdmgr->mctx = NULL;
+	isc_mem_attach(mctx, &httpdmgr->mctx);
+	httpdmgr->sock = NULL;
+	isc_socket_attach(sock, &httpdmgr->sock);
+	httpdmgr->task = NULL;
+	isc_task_attach(task, &httpdmgr->task);
+	httpdmgr->timermgr = tmgr; /* XXXMLG no attach function? */
+	httpdmgr->client_ok = client_ok;
+	httpdmgr->ondestroy = ondestroy;
+	httpdmgr->cb_arg = cb_arg;
+	httpdmgr->flags = 0;
 
-	ISC_LIST_INIT(httpd->running);
-	ISC_LIST_INIT(httpd->urls);
+	ISC_LIST_INIT(httpdmgr->running);
+	ISC_LIST_INIT(httpdmgr->urls);
 
 	/* XXXMLG ignore errors on isc_socket_listen() */
 	result = isc_socket_listen(sock, SOMAXCONN);
@@ -301,28 +284,27 @@ isc_httpdmgr_create(isc_mem_t *mctx, isc_socket_t *sock, isc_task_t *task,
 
 	(void)isc_socket_filter(sock, "httpready");
 
-	result = isc_socket_accept(sock, task, isc_httpd_accept, httpd);
+	result = isc_socket_accept(sock, task, isc_httpd_accept, httpdmgr);
 	if (result != ISC_R_SUCCESS)
 		goto cleanup;
 
-	httpd->render_404 = render_404;
-	httpd->render_500 = render_500;
+	httpdmgr->render_404 = render_404;
+	httpdmgr->render_500 = render_500;
 
-	*httpdp = httpd;
+	*httpdmgrp = httpdmgr;
 	return (ISC_R_SUCCESS);
 
   cleanup:
-	isc_task_detach(&httpd->task);
-	isc_socket_detach(&httpd->sock);
-	isc_mem_detach(&httpd->mctx);
-	(void)isc_mutex_destroy(&httpd->lock);
-	isc_mem_put(mctx, httpd, sizeof(isc_httpdmgr_t));
+	isc_task_detach(&httpdmgr->task);
+	isc_socket_detach(&httpdmgr->sock);
+	isc_mem_detach(&httpdmgr->mctx);
+	(void)isc_mutex_destroy(&httpdmgr->lock);
+	isc_mem_put(mctx, httpdmgr, sizeof(isc_httpdmgr_t));
 	return (result);
 }
 
 static void
-httpdmgr_destroy(isc_httpdmgr_t *httpdmgr)
-{
+httpdmgr_destroy(isc_httpdmgr_t *httpdmgr) {
 	isc_mem_t *mctx;
 	isc_httpdurl_t *url;
 
@@ -378,9 +360,75 @@ httpdmgr_destroy(isc_httpdmgr_t *httpdmgr)
 #define LENGTHOK(s) (httpd->recvbuf - (s) < (int)httpd->recvlen)
 #define BUFLENOK(s) (httpd->recvbuf - (s) < HTTP_RECVLEN)
 
-static isc_result_t
-process_request(isc_httpd_t *httpd, int length)
+/*
+ * Look for the given header in headers.
+ * If value is specified look for it terminated with a character in eov.
+ */
+static isc_boolean_t
+have_header(isc_httpd_t *httpd, const char *header, const char *value,
+	    const char *eov)
 {
+	char *cr, *nl, *h;
+	size_t hlen, vlen = 0;
+
+	h = httpd->headers;
+	hlen = strlen(header);
+	if (value != NULL) {
+		INSIST(eov != NULL);
+		vlen = strlen(value);
+	}
+
+	for (;;) {
+		if (strncasecmp(h, header, hlen) != 0) {
+			/*
+			 * Skip to next line;
+			 */
+			cr = strchr(h, '\r');
+			if (cr != NULL && cr[1] == '\n')
+				cr++;
+			nl = strchr(h, '\n');
+
+			/* last header? */
+			h = cr;
+			if (h == NULL || (nl != NULL && nl < h))
+				h = nl;
+			if (h == NULL)
+				return (ISC_FALSE);
+			h++;
+			continue;
+		}
+
+		if (value == NULL)
+			return (ISC_TRUE);
+
+		/*
+		 * Skip optional leading white space.
+		 */
+		h += hlen;
+		while (*h == ' ' || *h == '\t')
+			h++;
+		/*
+		 * Terminate token search on NULL or EOL.
+		 */
+		while (*h != 0 && *h != '\r' && *h != '\n') {
+			if (strncasecmp(h, value, vlen) == 0)
+				if (strchr(eov, h[vlen]) != NULL)
+					return (ISC_TRUE);
+			/*
+			 * Skip to next token.
+			 */
+			h += strcspn(h, eov);
+			if (h[0] == '\r' && h[1] == '\n')
+				h++;
+			if (h[0] != 0)
+				h++;
+		}
+		return (ISC_FALSE);
+	}
+}
+
+static isc_result_t
+process_request(isc_httpd_t *httpd, int length) {
 	char *s;
 	char *p;
 	int delim;
@@ -390,20 +438,26 @@ process_request(isc_httpd_t *httpd, int length)
 	httpd->recvlen += length;
 
 	httpd->recvbuf[httpd->recvlen] = 0;
+	httpd->headers = NULL;
 
 	/*
 	 * If we don't find a blank line in our buffer, return that we need
 	 * more data.
 	 */
 	s = strstr(httpd->recvbuf, "\r\n\r\n");
-	delim = 1;
+	delim = 2;
 	if (s == NULL) {
 		s = strstr(httpd->recvbuf, "\n\n");
-		delim = 2;
+		delim = 1;
 	}
 	if (s == NULL)
 		return (ISC_R_NOTFOUND);
 
+	/*
+	 * NUL terminate request at the blank line.
+	 */
+	s[delim] = 0;
+
 	/*
 	 * Determine if this is a POST or GET method.  Any other values will
 	 * cause an error to be returned.
@@ -462,7 +516,7 @@ process_request(isc_httpd_t *httpd, int length)
 	}
 
 	httpd->url = p;
-	p = s + delim;
+	p = s + 1;
 	s = p;
 
 	/*
@@ -477,7 +531,7 @@ process_request(isc_httpd_t *httpd, int length)
 
 	/*
 	 * Extract the HTTP/1.X protocol.  We will bounce on anything but
-	 * HTTP/1.1 for now.
+	 * HTTP/1.0 or HTTP/1.1 for now.
 	 */
 	while (LENGTHOK(s) && BUFLENOK(s) &&
 	       (*s != '\n' && *s != '\r' && *s != '\0'))
@@ -486,20 +540,35 @@ process_request(isc_httpd_t *httpd, int length)
 		return (ISC_R_NOTFOUND);
 	if (!BUFLENOK(s))
 		return (ISC_R_NOMEMORY);
+	/*
+	 * Check that we have the expected eol delimiter.
+	 */
+	if (strncmp(s, delim == 1 ? "\n" : "\r\n", delim) != 0)
+		return (ISC_R_RANGE);
 	*s = 0;
 	if ((strncmp(p, "HTTP/1.0", 8) != 0)
 	    && (strncmp(p, "HTTP/1.1", 8) != 0))
 		return (ISC_R_RANGE);
 	httpd->protocol = p;
-	p = s + 1;
+	p = s + delim;	/* skip past eol */
 	s = p;
 
-	if (strstr(s, "Connection: close") != NULL)
+	httpd->headers = s;
+
+	if (have_header(httpd, "Connection:", "close", ", \t\r\n"))
 		httpd->flags |= HTTPD_CLOSE;
 
-	if (strstr(s, "Host: ") != NULL)
+	if (have_header(httpd, "Host:", NULL, NULL))
 		httpd->flags |= HTTPD_FOUNDHOST;
 
+	if (strncmp(httpd->protocol, "HTTP/1.0", 8) == 0) {
+		if (have_header(httpd, "Connection:", "Keep-Alive",
+				", \t\r\n"))
+			httpd->flags |= HTTPD_KEEPALIVE;
+		else
+			httpd->flags |= HTTPD_CLOSE;
+	}
+
 	/*
 	 * Standards compliance hooks here.
 	 */
@@ -513,8 +582,7 @@ process_request(isc_httpd_t *httpd, int length)
 }
 
 static void
-isc_httpd_accept(isc_task_t *task, isc_event_t *ev)
-{
+isc_httpd_accept(isc_task_t *task, isc_event_t *ev) {
 	isc_result_t result;
 	isc_httpdmgr_t *httpdmgr = ev->ev_arg;
 	isc_httpd_t *httpd;
@@ -609,16 +677,18 @@ isc_httpd_accept(isc_task_t *task, isc_event_t *ev)
 }
 
 static isc_result_t
-render_404(const char *url, const char *querystring,
-	   void *arg,
+render_404(const char *url, isc_httpdurl_t *urlinfo,
+	   const char *querystring, const char *headers, void *arg,
 	   unsigned int *retcode, const char **retmsg,
 	   const char **mimetype, isc_buffer_t *b,
 	   isc_httpdfree_t **freecb, void **freecb_args)
 {
-	static char msg[] = "No such URL.";
+	static char msg[] = "No such URL.\r\n";
 
 	UNUSED(url);
+	UNUSED(urlinfo);
 	UNUSED(querystring);
+	UNUSED(headers);
 	UNUSED(arg);
 
 	*retcode = 404;
@@ -633,16 +703,18 @@ render_404(const char *url, const char *querystring,
 }
 
 static isc_result_t
-render_500(const char *url, const char *querystring,
-	   void *arg,
+render_500(const char *url, isc_httpdurl_t *urlinfo,
+	   const char *querystring, const char *headers, void *arg,
 	   unsigned int *retcode, const char **retmsg,
 	   const char **mimetype, isc_buffer_t *b,
 	   isc_httpdfree_t **freecb, void **freecb_args)
 {
-	static char msg[] = "Internal server failure.";
+	static char msg[] = "Internal server failure.\r\n";
 
 	UNUSED(url);
+	UNUSED(urlinfo);
 	UNUSED(querystring);
+	UNUSED(headers);
 	UNUSED(arg);
 
 	*retcode = 500;
@@ -657,15 +729,14 @@ render_500(const char *url, const char *querystring,
 }
 
 static void
-isc_httpd_recvdone(isc_task_t *task, isc_event_t *ev)
-{
+isc_httpd_recvdone(isc_task_t *task, isc_event_t *ev) {
 	isc_region_t r;
 	isc_result_t result;
 	isc_httpd_t *httpd = ev->ev_arg;
 	isc_socketevent_t *sev = (isc_socketevent_t *)ev;
 	isc_httpdurl_t *url;
 	isc_time_t now;
-	char datebuf[32];  /* Only need 30, but safety first */
+	char datebuf[ISC_FORMATHTTPTIMESTAMP_SIZE];
 
 	ENTER("recv");
 
@@ -710,8 +781,9 @@ isc_httpd_recvdone(isc_task_t *task, isc_event_t *ev)
 		url = ISC_LIST_NEXT(url, link);
 	}
 	if (url == NULL)
-		result = httpd->mgr->render_404(httpd->url, httpd->querystring,
-						NULL,
+		result = httpd->mgr->render_404(httpd->url, NULL,
+						httpd->querystring,
+						NULL, NULL,
 						&httpd->retcode,
 						&httpd->retmsg,
 						&httpd->mimetype,
@@ -719,14 +791,18 @@ isc_httpd_recvdone(isc_task_t *task, isc_event_t *ev)
 						&httpd->freecb,
 						&httpd->freecb_arg);
 	else
-		result = url->action(httpd->url, httpd->querystring,
+		result = url->action(httpd->url, url,
+				     httpd->querystring,
+				     httpd->headers,
 				     url->action_arg,
 				     &httpd->retcode, &httpd->retmsg,
 				     &httpd->mimetype, &httpd->bodybuffer,
 				     &httpd->freecb, &httpd->freecb_arg);
 	if (result != ISC_R_SUCCESS) {
-		result = httpd->mgr->render_500(httpd->url, httpd->querystring,
-						NULL, &httpd->retcode,
+		result = httpd->mgr->render_500(httpd->url, url,
+						httpd->querystring,
+						NULL, NULL,
+						&httpd->retcode,
 						&httpd->retmsg,
 						&httpd->mimetype,
 						&httpd->bodybuffer,
@@ -736,12 +812,24 @@ isc_httpd_recvdone(isc_task_t *task, isc_event_t *ev)
 	}
 
 	isc_httpd_response(httpd);
+	if ((httpd->flags & HTTPD_KEEPALIVE) != 0)
+		isc_httpd_addheader(httpd, "Connection", "Keep-Alive");
 	isc_httpd_addheader(httpd, "Content-Type", httpd->mimetype);
 	isc_httpd_addheader(httpd, "Date", datebuf);
 	isc_httpd_addheader(httpd, "Expires", datebuf);
-	isc_httpd_addheader(httpd, "Last-Modified", datebuf);
-	isc_httpd_addheader(httpd, "Pragma: no-cache", NULL);
-	isc_httpd_addheader(httpd, "Cache-Control: no-cache", NULL);
+
+	if (url != NULL && url->isstatic) {
+		char loadbuf[ISC_FORMATHTTPTIMESTAMP_SIZE];
+		isc_time_formathttptimestamp(&url->loadtime,
+					     loadbuf, sizeof(loadbuf));
+		isc_httpd_addheader(httpd, "Last-Modified", loadbuf);
+		isc_httpd_addheader(httpd, "Cache-Control: public", NULL);
+	} else {
+		isc_httpd_addheader(httpd, "Last-Modified", datebuf);
+		isc_httpd_addheader(httpd, "Pragma: no-cache", NULL);
+		isc_httpd_addheader(httpd, "Cache-Control: no-cache", NULL);
+	}
+
 	isc_httpd_addheader(httpd, "Server: libisc", NULL);
 	isc_httpd_addheaderuint(httpd, "Content-Length",
 				isc_buffer_usedlength(&httpd->bodybuffer));
@@ -766,8 +854,7 @@ isc_httpd_recvdone(isc_task_t *task, isc_event_t *ev)
 }
 
 void
-isc_httpdmgr_shutdown(isc_httpdmgr_t **httpdmgrp)
-{
+isc_httpdmgr_shutdown(isc_httpdmgr_t **httpdmgrp) {
 	isc_httpdmgr_t *httpdmgr;
 	isc_httpd_t *httpd;
 	httpdmgr = *httpdmgrp;
@@ -794,8 +881,7 @@ isc_httpdmgr_shutdown(isc_httpdmgr_t **httpdmgrp)
 }
 
 static isc_result_t
-grow_headerspace(isc_httpd_t *httpd)
-{
+grow_headerspace(isc_httpd_t *httpd) {
 	char *newspace;
 	unsigned int newlen;
 	isc_region_t r;
@@ -816,8 +902,7 @@ grow_headerspace(isc_httpd_t *httpd)
 }
 
 isc_result_t
-isc_httpd_response(isc_httpd_t *httpd)
-{
+isc_httpd_response(isc_httpd_t *httpd) {
 	isc_result_t result;
 	unsigned int needlen;
 
@@ -831,8 +916,10 @@ isc_httpd_response(isc_httpd_t *httpd)
 			return (result);
 	}
 
-	sprintf(isc_buffer_used(&httpd->headerbuffer), "%s %03d %s\r\n",
-		httpd->protocol, httpd->retcode, httpd->retmsg);
+	snprintf(isc_buffer_used(&httpd->headerbuffer),
+		 (int)isc_buffer_availablelength(&httpd->headerbuffer),
+		 "%s %03u %s\r\n", httpd->protocol, httpd->retcode,
+		 httpd->retmsg);
 	isc_buffer_add(&httpd->headerbuffer, needlen);
 
 	return (ISC_R_SUCCESS);
@@ -857,11 +944,13 @@ isc_httpd_addheader(isc_httpd_t *httpd, const char *name,
 	}
 
 	if (val != NULL)
-		sprintf(isc_buffer_used(&httpd->headerbuffer),
-			"%s: %s\r\n", name, val);
+		snprintf(isc_buffer_used(&httpd->headerbuffer),
+			 isc_buffer_availablelength(&httpd->headerbuffer),
+			 "%s: %s\r\n", name, val);
 	else
-		sprintf(isc_buffer_used(&httpd->headerbuffer),
-			"%s\r\n", name);
+		snprintf(isc_buffer_used(&httpd->headerbuffer),
+			 isc_buffer_availablelength(&httpd->headerbuffer),
+			 "%s\r\n", name);
 
 	isc_buffer_add(&httpd->headerbuffer, needlen);
 
@@ -869,8 +958,7 @@ isc_httpd_addheader(isc_httpd_t *httpd, const char *name,
 }
 
 isc_result_t
-isc_httpd_endheaders(isc_httpd_t *httpd)
-{
+isc_httpd_endheaders(isc_httpd_t *httpd) {
 	isc_result_t result;
 
 	while (isc_buffer_availablelength(&httpd->headerbuffer) < 2) {
@@ -879,7 +967,8 @@ isc_httpd_endheaders(isc_httpd_t *httpd)
 			return (result);
 	}
 
-	sprintf(isc_buffer_used(&httpd->headerbuffer), "\r\n");
+	snprintf(isc_buffer_used(&httpd->headerbuffer),
+		 isc_buffer_availablelength(&httpd->headerbuffer), "\r\n");
 	isc_buffer_add(&httpd->headerbuffer, 2);
 
 	return (ISC_R_SUCCESS);
@@ -891,7 +980,7 @@ isc_httpd_addheaderuint(isc_httpd_t *httpd, const char *name, int val) {
 	unsigned int needlen;
 	char buf[sizeof "18446744073709551616"];
 
-	sprintf(buf, "%d", val);
+	snprintf(buf, sizeof(buf), "%d", val);
 
 	needlen = strlen(name); /* name itself */
 	needlen += 2 + strlen(buf); /* :<space> and val */
@@ -903,8 +992,9 @@ isc_httpd_addheaderuint(isc_httpd_t *httpd, const char *name, int val) {
 			return (result);
 	}
 
-	sprintf(isc_buffer_used(&httpd->headerbuffer),
-		"%s: %s\r\n", name, buf);
+	snprintf(isc_buffer_used(&httpd->headerbuffer),
+		 isc_buffer_availablelength(&httpd->headerbuffer),
+		 "%s: %s\r\n", name, buf);
 
 	isc_buffer_add(&httpd->headerbuffer, needlen);
 
@@ -912,8 +1002,7 @@ isc_httpd_addheaderuint(isc_httpd_t *httpd, const char *name, int val) {
 }
 
 static void
-isc_httpd_senddone(isc_task_t *task, isc_event_t *ev)
-{
+isc_httpd_senddone(isc_task_t *task, isc_event_t *ev) {
 	isc_httpd_t *httpd = ev->ev_arg;
 	isc_region_t r;
 	isc_socketevent_t *sev = (isc_socketevent_t *)ev;
@@ -976,8 +1065,7 @@ out:
 }
 
 static void
-reset_client(isc_httpd_t *httpd)
-{
+reset_client(isc_httpd_t *httpd) {
 	/*
 	 * Catch errors here.  We MUST be in RECV mode, and we MUST NOT have
 	 * any outstanding buffers.  If we have buffers, we have a leak.
@@ -988,6 +1076,7 @@ reset_client(isc_httpd_t *httpd)
 
 	httpd->recvbuf[0] = 0;
 	httpd->recvlen = 0;
+	httpd->headers = NULL;
 	httpd->method = ISC_HTTPD_METHODUNKNOWN;
 	httpd->url = NULL;
 	httpd->querystring = NULL;
@@ -1001,6 +1090,14 @@ reset_client(isc_httpd_t *httpd)
 isc_result_t
 isc_httpdmgr_addurl(isc_httpdmgr_t *httpdmgr, const char *url,
 		    isc_httpdaction_t *func, void *arg)
+{
+	return (isc_httpdmgr_addurl2(httpdmgr, url, ISC_FALSE, func, arg));
+}
+
+isc_result_t
+isc_httpdmgr_addurl2(isc_httpdmgr_t *httpdmgr, const char *url,
+		     isc_boolean_t isstatic,
+		     isc_httpdaction_t *func, void *arg)
 {
 	isc_httpdurl_t *item;
 
@@ -1021,6 +1118,9 @@ isc_httpdmgr_addurl(isc_httpdmgr_t *httpdmgr, const char *url,
 
 	item->action = func;
 	item->action_arg = arg;
+	item->isstatic = isstatic;
+	isc_time_now(&item->loadtime);
+
 	ISC_LINK_INIT(item, link);
 	ISC_LIST_APPEND(httpdmgr->urls, item, link);
 
