@@ -1,6 +1,5 @@
 /*
- * Copyright (C) 2004-2012  Internet Systems Consortium, Inc. ("ISC")
- * Copyright (C) 1998-2003  Internet Software Consortium.
+ * Copyright (C) Internet Systems Consortium, Inc. ("ISC")
  *
  * Permission to use, copy, modify, and/or distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -15,8 +14,6 @@
  * PERFORMANCE OF THIS SOFTWARE.
  */
 
-/* $Id$ */
-
 /*! \file
  * \author Principal Author: Bob Halley
  */
@@ -34,6 +31,7 @@
 #include <isc/mem.h>
 #include <isc/msgs.h>
 #include <isc/platform.h>
+#include <isc/print.h>
 #include <isc/string.h>
 #include <isc/task.h>
 #include <isc/thread.h>
@@ -156,6 +154,14 @@ struct isc__taskmgr {
 	isc_boolean_t			pause_requested;
 	isc_boolean_t			exclusive_requested;
 	isc_boolean_t			exiting;
+
+	/*
+	 * Multiple threads can read/write 'excl' at the same time, so we need
+	 * to protect the access.  We can't use 'lock' since isc_task_detach()
+	 * will try to acquire it.
+	 */
+	isc_mutex_t			excl_lock;
+	isc__task_t			*excl;
 #ifdef USE_SHARED_MANAGER
 	unsigned int			refs;
 #endif /* ISC_PLATFORM_USETHREADS */
@@ -207,7 +213,7 @@ isc__task_unsend(isc_task_t *task, void *sender, isc_eventtype_t type,
 		 void *tag, isc_eventlist_t *events);
 ISC_TASKFUNC_SCOPE isc_result_t
 isc__task_onshutdown(isc_task_t *task0, isc_taskaction_t action,
-		     const void *arg);
+		     void *arg);
 ISC_TASKFUNC_SCOPE void
 isc__task_shutdown(isc_task_t *task0);
 ISC_TASKFUNC_SCOPE void
@@ -225,6 +231,10 @@ isc__taskmgr_create(isc_mem_t *mctx, unsigned int workers,
 		    unsigned int default_quantum, isc_taskmgr_t **managerp);
 ISC_TASKFUNC_SCOPE void
 isc__taskmgr_destroy(isc_taskmgr_t **managerp);
+ISC_TASKFUNC_SCOPE void
+isc__taskmgr_setexcltask(isc_taskmgr_t *mgr0, isc_task_t *task0);
+ISC_TASKFUNC_SCOPE isc_result_t
+isc__taskmgr_excltask(isc_taskmgr_t *mgr0, isc_task_t **taskp);
 ISC_TASKFUNC_SCOPE isc_result_t
 isc__task_beginexclusive(isc_task_t *task);
 ISC_TASKFUNC_SCOPE void
@@ -286,7 +296,9 @@ static isc_taskmgrmethods_t taskmgrmethods = {
 	isc__taskmgr_destroy,
 	isc__taskmgr_setmode,
 	isc__taskmgr_mode,
-	isc__task_create
+	isc__task_create,
+	isc__taskmgr_setexcltask,
+	isc__taskmgr_excltask
 };
 
 /***
@@ -536,6 +548,7 @@ task_send(isc__task_t *task, isc_event_t **eventp) {
 	REQUIRE(event != NULL);
 	REQUIRE(event->ev_type > 0);
 	REQUIRE(task->state != task_state_done);
+	REQUIRE(!ISC_LINK_LINKED(event, ev_ratelink));
 
 	XTRACE("task_send");
 
@@ -692,6 +705,7 @@ isc__task_purgerange(isc_task_t *task0, void *sender, isc_eventtype_t first,
 
 	for (event = HEAD(events); event != NULL; event = next_event) {
 		next_event = NEXT(event, ev_link);
+		ISC_LIST_UNLINK(events, event, ev_link);
 		isc_event_free(&event);
 	}
 
@@ -789,7 +803,7 @@ isc__task_unsend(isc_task_t *task, void *sender, isc_eventtype_t type,
 
 ISC_TASKFUNC_SCOPE isc_result_t
 isc__task_onshutdown(isc_task_t *task0, isc_taskaction_t action,
-		     const void *arg)
+		     void *arg)
 {
 	isc__task_t *task = (isc__task_t *)task0;
 	isc_boolean_t disallowed = ISC_FALSE;
@@ -870,8 +884,7 @@ isc__task_setname(isc_task_t *task0, const char *name, void *tag) {
 	REQUIRE(VALID_TASK(task));
 
 	LOCK(&task->lock);
-	memset(task->name, 0, sizeof(task->name));
-	strncpy(task->name, name, sizeof(task->name) - 1);
+	strlcpy(task->name, name, sizeof(task->name));
 	task->tag = tag;
 	UNLOCK(&task->lock);
 }
@@ -1300,6 +1313,7 @@ manager_free(isc__taskmgr_t *manager) {
 	isc_mem_free(manager->mctx, manager->threads);
 #endif /* USE_WORKER_THREADS */
 	DESTROYLOCK(&manager->lock);
+	DESTROYLOCK(&manager->excl_lock);
 	manager->common.impmagic = 0;
 	manager->common.magic = 0;
 	mctx = manager->mctx;
@@ -1352,6 +1366,11 @@ isc__taskmgr_create(isc_mem_t *mctx, unsigned int workers,
 	result = isc_mutex_init(&manager->lock);
 	if (result != ISC_R_SUCCESS)
 		goto cleanup_mgr;
+	result = isc_mutex_init(&manager->excl_lock);
+	if (result != ISC_R_SUCCESS) {
+		DESTROYLOCK(&manager->lock);
+		goto cleanup_mgr;
+	}
 
 #ifdef USE_WORKER_THREADS
 	manager->workers = 0;
@@ -1396,6 +1415,7 @@ isc__taskmgr_create(isc_mem_t *mctx, unsigned int workers,
 	manager->exclusive_requested = ISC_FALSE;
 	manager->pause_requested = ISC_FALSE;
 	manager->exiting = ISC_FALSE;
+	manager->excl = NULL;
 
 	isc_mem_attach(mctx, &manager->mctx);
 
@@ -1408,6 +1428,10 @@ isc__taskmgr_create(isc_mem_t *mctx, unsigned int workers,
 		if (isc_thread_create(run, manager,
 				      &manager->threads[manager->workers]) ==
 		    ISC_R_SUCCESS) {
+			char name[16];	/* thread name limit on Linux */
+			snprintf(name, sizeof(name), "isc-worker%04u", i);
+			isc_thread_setname(manager->threads[manager->workers],
+					   name);
 			manager->workers++;
 			started++;
 		}
@@ -1479,6 +1503,14 @@ isc__taskmgr_destroy(isc_taskmgr_t **managerp) {
 	 * that the startup thread is sleeping on.
 	 */
 
+	/*
+	 * Detach the exclusive task before acquiring the manager lock
+	 */
+	LOCK(&manager->excl_lock);
+	if (manager->excl != NULL)
+		isc__task_detach((isc_task_t **) &manager->excl);
+	UNLOCK(&manager->excl_lock);
+
 	/*
 	 * Unlike elsewhere, we're going to hold this lock a long time.
 	 * We need to do so, because otherwise the list of tasks could
@@ -1608,11 +1640,11 @@ isc__taskmgr_dispatch(isc_taskmgr_t *manager0) {
 ISC_TASKFUNC_SCOPE void
 isc__taskmgr_pause(isc_taskmgr_t *manager0) {
 	isc__taskmgr_t *manager = (isc__taskmgr_t *)manager0;
+	manager->pause_requested = ISC_TRUE;
 	LOCK(&manager->lock);
 	while (manager->tasks_running > 0) {
 		WAIT(&manager->paused, &manager->lock);
 	}
-	manager->pause_requested = ISC_TRUE;
 	UNLOCK(&manager->lock);
 }
 
@@ -1629,12 +1661,47 @@ isc__taskmgr_resume(isc_taskmgr_t *manager0) {
 }
 #endif /* USE_WORKER_THREADS */
 
+ISC_TASKFUNC_SCOPE void
+isc__taskmgr_setexcltask(isc_taskmgr_t *mgr0, isc_task_t *task0) {
+	isc__taskmgr_t *mgr = (isc__taskmgr_t *) mgr0;
+	isc__task_t *task = (isc__task_t *) task0;
+
+	REQUIRE(VALID_MANAGER(mgr));
+	REQUIRE(VALID_TASK(task));
+	LOCK(&mgr->excl_lock);
+	if (mgr->excl != NULL)
+		isc__task_detach((isc_task_t **) &mgr->excl);
+	isc__task_attach(task0, (isc_task_t **) &mgr->excl);
+	UNLOCK(&mgr->excl_lock);
+}
+
+ISC_TASKFUNC_SCOPE isc_result_t
+isc__taskmgr_excltask(isc_taskmgr_t *mgr0, isc_task_t **taskp) {
+	isc__taskmgr_t *mgr = (isc__taskmgr_t *) mgr0;
+	isc_result_t result = ISC_R_SUCCESS;
+
+	REQUIRE(VALID_MANAGER(mgr));
+	REQUIRE(taskp != NULL && *taskp == NULL);
+
+	LOCK(&mgr->excl_lock);
+	if (mgr->excl != NULL)
+		isc__task_attach((isc_task_t *) mgr->excl, taskp);
+	else
+		result = ISC_R_NOTFOUND;
+	UNLOCK(&mgr->excl_lock);
+
+	return (result);
+}
+
 ISC_TASKFUNC_SCOPE isc_result_t
 isc__task_beginexclusive(isc_task_t *task0) {
 #ifdef USE_WORKER_THREADS
 	isc__task_t *task = (isc__task_t *)task0;
 	isc__taskmgr_t *manager = task->manager;
+
 	REQUIRE(task->state == task_state_running);
+	/* XXX: Require task == manager->excl? */
+
 	LOCK(&manager->lock);
 	if (manager->exclusive_requested) {
 		UNLOCK(&manager->lock);
@@ -1708,7 +1775,7 @@ isc__task_privilege(isc_task_t *task0) {
 
 #ifdef USE_SOCKETIMPREGISTER
 isc_result_t
-isc__task_register() {
+isc__task_register(void) {
 	return (isc_task_register(isc__taskmgr_create));
 }
 #endif
@@ -1723,10 +1790,12 @@ isc_task_exiting(isc_task_t *t) {
 
 
 #if defined(HAVE_LIBXML2) && defined(BIND9)
-void
+#define TRY0(a) do { xmlrc = (a); if (xmlrc < 0) goto error; } while(0)
+int
 isc_taskmgr_renderxml(isc_taskmgr_t *mgr0, xmlTextWriterPtr writer) {
 	isc__taskmgr_t *mgr = (isc__taskmgr_t *)mgr0;
-	isc__task_t *task;
+	isc__task_t *task = NULL;
+	int xmlrc;
 
 	LOCK(&mgr->lock);
 
@@ -1734,72 +1803,82 @@ isc_taskmgr_renderxml(isc_taskmgr_t *mgr0, xmlTextWriterPtr writer) {
 	 * Write out the thread-model, and some details about each depending
 	 * on which type is enabled.
 	 */
-	xmlTextWriterStartElement(writer, ISC_XMLCHAR "thread-model");
+	TRY0(xmlTextWriterStartElement(writer, ISC_XMLCHAR "thread-model"));
 #ifdef ISC_PLATFORM_USETHREADS
-	xmlTextWriterStartElement(writer, ISC_XMLCHAR "type");
-	xmlTextWriterWriteString(writer, ISC_XMLCHAR "threaded");
-	xmlTextWriterEndElement(writer); /* type */
+	TRY0(xmlTextWriterStartElement(writer, ISC_XMLCHAR "type"));
+	TRY0(xmlTextWriterWriteString(writer, ISC_XMLCHAR "threaded"));
+	TRY0(xmlTextWriterEndElement(writer)); /* type */
 
-	xmlTextWriterStartElement(writer, ISC_XMLCHAR "worker-threads");
-	xmlTextWriterWriteFormatString(writer, "%d", mgr->workers);
-	xmlTextWriterEndElement(writer); /* worker-threads */
+	TRY0(xmlTextWriterStartElement(writer, ISC_XMLCHAR "worker-threads"));
+	TRY0(xmlTextWriterWriteFormatString(writer, "%d", mgr->workers));
+	TRY0(xmlTextWriterEndElement(writer)); /* worker-threads */
 #else /* ISC_PLATFORM_USETHREADS */
-	xmlTextWriterStartElement(writer, ISC_XMLCHAR "type");
-	xmlTextWriterWriteString(writer, ISC_XMLCHAR "non-threaded");
-	xmlTextWriterEndElement(writer); /* type */
+	TRY0(xmlTextWriterStartElement(writer, ISC_XMLCHAR "type"));
+	TRY0(xmlTextWriterWriteString(writer, ISC_XMLCHAR "non-threaded"));
+	TRY0(xmlTextWriterEndElement(writer)); /* type */
 
-	xmlTextWriterStartElement(writer, ISC_XMLCHAR "references");
-	xmlTextWriterWriteFormatString(writer, "%d", mgr->refs);
-	xmlTextWriterEndElement(writer); /* references */
+	TRY0(xmlTextWriterStartElement(writer, ISC_XMLCHAR "references"));
+	TRY0(xmlTextWriterWriteFormatString(writer, "%d", mgr->refs));
+	TRY0(xmlTextWriterEndElement(writer)); /* references */
 #endif /* ISC_PLATFORM_USETHREADS */
 
-	xmlTextWriterStartElement(writer, ISC_XMLCHAR "default-quantum");
-	xmlTextWriterWriteFormatString(writer, "%d", mgr->default_quantum);
-	xmlTextWriterEndElement(writer); /* default-quantum */
+	TRY0(xmlTextWriterStartElement(writer, ISC_XMLCHAR "default-quantum"));
+	TRY0(xmlTextWriterWriteFormatString(writer, "%d",
+					    mgr->default_quantum));
+	TRY0(xmlTextWriterEndElement(writer)); /* default-quantum */
 
-	xmlTextWriterStartElement(writer, ISC_XMLCHAR "tasks-running");
-	xmlTextWriterWriteFormatString(writer, "%d", mgr->tasks_running);
-	xmlTextWriterEndElement(writer); /* tasks-running */
+	TRY0(xmlTextWriterStartElement(writer, ISC_XMLCHAR "tasks-running"));
+	TRY0(xmlTextWriterWriteFormatString(writer, "%d", mgr->tasks_running));
+	TRY0(xmlTextWriterEndElement(writer)); /* tasks-running */
 
-	xmlTextWriterEndElement(writer); /* thread-model */
+	TRY0(xmlTextWriterEndElement(writer)); /* thread-model */
 
-	xmlTextWriterStartElement(writer, ISC_XMLCHAR "tasks");
+	TRY0(xmlTextWriterStartElement(writer, ISC_XMLCHAR "tasks"));
 	task = ISC_LIST_HEAD(mgr->tasks);
 	while (task != NULL) {
 		LOCK(&task->lock);
-		xmlTextWriterStartElement(writer, ISC_XMLCHAR "task");
+		TRY0(xmlTextWriterStartElement(writer, ISC_XMLCHAR "task"));
 
 		if (task->name[0] != 0) {
-			xmlTextWriterStartElement(writer, ISC_XMLCHAR "name");
-			xmlTextWriterWriteFormatString(writer, "%s",
-						       task->name);
-			xmlTextWriterEndElement(writer); /* name */
+			TRY0(xmlTextWriterStartElement(writer,
+						       ISC_XMLCHAR "name"));
+			TRY0(xmlTextWriterWriteFormatString(writer, "%s",
+						       task->name));
+			TRY0(xmlTextWriterEndElement(writer)); /* name */
 		}
 
-		xmlTextWriterStartElement(writer, ISC_XMLCHAR "references");
-		xmlTextWriterWriteFormatString(writer, "%d", task->references);
-		xmlTextWriterEndElement(writer); /* references */
+		TRY0(xmlTextWriterStartElement(writer,
+					       ISC_XMLCHAR "references"));
+		TRY0(xmlTextWriterWriteFormatString(writer, "%d",
+						    task->references));
+		TRY0(xmlTextWriterEndElement(writer)); /* references */
 
-		xmlTextWriterStartElement(writer, ISC_XMLCHAR "id");
-		xmlTextWriterWriteFormatString(writer, "%p", task);
-		xmlTextWriterEndElement(writer); /* id */
+		TRY0(xmlTextWriterStartElement(writer, ISC_XMLCHAR "id"));
+		TRY0(xmlTextWriterWriteFormatString(writer, "%p", task));
+		TRY0(xmlTextWriterEndElement(writer)); /* id */
 
-		xmlTextWriterStartElement(writer, ISC_XMLCHAR "state");
-		xmlTextWriterWriteFormatString(writer, "%s",
-					       statenames[task->state]);
-		xmlTextWriterEndElement(writer); /* state */
+		TRY0(xmlTextWriterStartElement(writer, ISC_XMLCHAR "state"));
+		TRY0(xmlTextWriterWriteFormatString(writer, "%s",
+					       statenames[task->state]));
+		TRY0(xmlTextWriterEndElement(writer)); /* state */
 
-		xmlTextWriterStartElement(writer, ISC_XMLCHAR "quantum");
-		xmlTextWriterWriteFormatString(writer, "%d", task->quantum);
-		xmlTextWriterEndElement(writer); /* quantum */
+		TRY0(xmlTextWriterStartElement(writer, ISC_XMLCHAR "quantum"));
+		TRY0(xmlTextWriterWriteFormatString(writer, "%d",
+						    task->quantum));
+		TRY0(xmlTextWriterEndElement(writer)); /* quantum */
 
-		xmlTextWriterEndElement(writer);
+		TRY0(xmlTextWriterEndElement(writer));
 
 		UNLOCK(&task->lock);
 		task = ISC_LIST_NEXT(task, link);
 	}
-	xmlTextWriterEndElement(writer); /* tasks */
+	TRY0(xmlTextWriterEndElement(writer)); /* tasks */
 
+ error:
+	if (task != NULL)
+		UNLOCK(&task->lock);
 	UNLOCK(&mgr->lock);
+
+	return (xmlrc);
 }
 #endif /* HAVE_LIBXML2 && BIND9 */
